/************************************************************************
 *   AustHex IRCD - AustNet Internet Relay Chat Daemon, src/module.c
 *   Copyright (C) 1998 Roger Yerramsetti
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Virtual world written by Roger 'RogerY' Yerramsetti <rogery@austnet.org> */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "../include/struct.h"

#define	HASHVAL_TOTAL 	30011
#define	HASHVAL_PARTIAL	211

#define	MAXVIRTSIZE	(3 + 5 + 1)

#define	aust1 "NOTICE AUTH :                     __             __\n"
#define	aust2 "NOTICE AUTH :   ____ ___  _______/ /_____  ___  / /_      _ _ _  . _ _ _\n"
#define	aust3 "NOTICE AUTH :  / __ `/ / / / ___/ __/ __ \\/ _ \\/ __/     _)(-| \\/|(_(-_)\n"
#define	aust4 "NOTICE AUTH : / /_/ / /_/ (__  ) /_/ / / /  __/ /__     compatible daemon\n"
#define	aust5 "NOTICE AUTH : \\__,_/\\__,_/____/\\__/_/ /_/\\___/\\___/      austhex.servd"UPTO"\n"
#define aust6 "NOTICE AUTH :        http://www.austnet.org\n"
#define aust7 "NOTICE AUTH :\n"

extern	aClient me;
extern	int	seed;
int	match(char *, char *), find_exception(char *);

extern unsigned char tolowertab[];

#undef tolower
#define tolower(c) (tolowertab[(u_char)(c)])

int 
SetClient(sptr)
    aClient *sptr;
{
    SetUser(sptr);

    if(MyConnect(sptr))
    {
	send(sptr->fd,aust1,52,0);
 	send(sptr->fd,aust2,73,0);
	send(sptr->fd,aust3,73,0);
	send(sptr->fd,aust4,74,0);
 	send(sptr->fd,aust5,73,0);
	send(sptr->fd,aust6,44,0);
	send(sptr->fd,aust7,14,0);
    }
}

unsigned
z(char *s)
{
    unsigned int i;

    for(i=0;*s;s++)
	i=seed*i+*s;

    return i;
}

int
str2array(char **pparv, char *string, char *delim)
{
    char *tok;
    int pparc=0;

    tok=strtok(string,delim);
    while(tok != NULL)
    {
	pparv[pparc++]=tok;
	tok=strtok(NULL,delim);
    }

    return pparc;
}

void
make_virthost(char *curr, char *new)
{
/* Virtual world written by Roger 'RogerY' Yerramsetti <rogery@austnet.org> */
    unsigned hash_total = z(curr) % HASHVAL_TOTAL, hash_partial; 
    char     destroy[HOSTLEN+1], *parv[HOSTLEN+1], tmpnew[HOSTLEN], *ptr;
    int      parc=0, len=0, overflow=0, exception=0;

    strncpyzt(destroy, curr, HOSTLEN);

    len = strlen(destroy);

    if ((len + MAXVIRTSIZE) > HOSTLEN)
    {
	overflow = (len + MAXVIRTSIZE) - HOSTLEN;

	ptr = &destroy[overflow];
    }
    else
	ptr = &destroy[0];

    parc = str2array(parv, ptr, ".");

    if (parc == 4)
    {
	len = strlen(parv[3]);

	if (strchr("0123456789", parv[3][len-1]))
	{
	    hash_partial = (z(parv[0])+z(parv[1])+z(parv[2])) % HASHVAL_PARTIAL;
	    sprintf(tmpnew, "%s.%s.%u.%u",
		parv[0], parv[1], hash_partial, hash_total);

	    strncpyzt(new, tmpnew, HOSTLEN);
	    return;
	}
    }

    if (parc == 2)
    {
	sprintf(tmpnew, "vw-%u.%s.%s", 
	    hash_total, parv[parc-2], parv[parc-1]);

	strncpyzt(new, tmpnew, HOSTLEN);
	return;
    }

    len = strlen(parv[parc-1]);

    if ((len==2) && (find_exception(parv[parc-1])))
	exception = 1;

    if ((len == 2) && (exception == 0))
    {
	sprintf(tmpnew, "vw-%u.%s.%s.%s",
	    hash_total, parv[parc-3], parv[parc-2], parv[parc-1]);

	strncpyzt(new, tmpnew, HOSTLEN);
	return;
    }
	
    sprintf(tmpnew, "vw-%u.%s.%s",
	hash_total, parv[parc-2], parv[parc-1]);

    strncpyzt(new, tmpnew, HOSTLEN);
    return;
}
